@@ -1,27 +1,7 @@
 /*
-    Copyright (c) 2017 Volker Krause <vkrause@kde.org>
-
-    Permission is hereby granted, free of charge, to any person
-    obtaining a copy of this software and associated documentation
-    files (the "Software"), to deal in the Software without
-    restriction, including without limitation the rights to use,
-    copy, modify, merge, publish, distribute, sublicense, and/or sell
-    copies of the Software, and to permit persons to whom the
-    Software is furnished to do so, subject to the following
-    conditions:
-
-    The above copyright notice and this permission notice shall be
-    included in all copies or substantial portions of the Software.
-
-    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-    OTHER DEALINGS IN THE SOFTWARE.
+    SPDX-FileCopyrightText: 2017 Volker Krause <vkrause@kde.org>
 
+    SPDX-License-Identifier: MIT
 */
 
 #include "aztecbarcode.h"
@@ -162,17 +142,15 @@ QImage AztecBarcode::paintImage(const QSizeF& size)
         paintCompactGrid(&img);
         paintCompactData(&img, encodedData, layerCount);
         paintCompactModeMessage(&img, modeMsg);
-        return cropAndScaleCompact(&img, layerCount, std::max(size.width(), size.height()));
+        return cropAndScaleCompact(&img, layerCount, std::min(size.width(), size.height()));
     } else {
         QImage img(FullMaxSize, FullMaxSize, QImage::Format_RGB32);
         img.fill(backgroundColor());
         paintFullGrid(&img);
         paintFullData(&img, encodedData, layerCount);
         paintFullModeMessage(&img, modeMsg);
-        return cropAndScaleFull(&img, layerCount, std::max(size.width(), size.height()));
+        return cropAndScaleFull(&img, layerCount, std::min(size.width(), size.height()));
     }
-
-    return {};
 }
 
 // code points and encoding modes for each of the first 127 ASCII characters, the rest is encoded in Binary mode
@@ -343,7 +321,6 @@ static const struct {
     { ',',   ' ', { 4, Punct } }, // , SP
     { ':',   ' ', { 5, Punct } }  // : SP
 };
-static const int aztec_code_double_symbols_count = sizeof(aztec_code_double_symbols) / sizeof(aztec_code_double_symbols[0]);
 
 static const int aztec_code_size[] = { 0, 5, 5, 5, 5, 4, 8 };
 Q_STATIC_ASSERT(sizeof(aztec_code_size) / sizeof(int) == MODE_COUNT);
@@ -375,7 +352,7 @@ static const aztec_code_t aztec_latch_codes[MODE_COUNT - 1][MODE_COUNT] = {
     { { 0, NoMode }, { 0, NoMode }, { 0, NoMode }, { 0, NoMode }, { 0, NoMode }, { 0, NoMode }, { 0, NoMode } },
     { { 0, NoMode }, { 0, NoMode }, { 28, Lower }, { 29, Mixed }, { 29, Mixed }, { 30, Digit }, { 31, Binary } },
     { { 0, NoMode }, { 30, Digit }, { 0, NoMode }, { 29, Mixed }, { 29, Mixed }, { 30, Digit }, { 31, Binary } },
-    { { 0, NoMode }, { 30, Digit }, { 28, Lower }, { 0, NoMode }, { 30, Punct }, { 28, Lower }, { 31, Binary } },
+    { { 0, NoMode }, { 29, Upper }, { 28, Lower }, { 0, NoMode }, { 30, Punct }, { 28, Lower }, { 31, Binary } },
     { { 0, NoMode }, { 31, Upper }, { 31, Upper }, { 31, Upper }, { 0, NoMode }, { 31, Upper }, { 31, Upper } },
     { { 0, NoMode }, { 14, Upper }, { 14, Upper }, { 14, Upper }, { 14, Upper }, { 0, NoMode }, { 14, Upper } }
 };
@@ -484,8 +461,7 @@ BitVector AztecBarcode::aztecEncode(const QByteArray &data) const
         if (i < data.size() - 1) {
             const uint8_t c2 = data.at(i + 1);
             bool found = false;
-            for (int j = 0; j < aztec_code_double_symbols_count; ++j) {
-                const auto dblCode = aztec_code_double_symbols[j];
+            for (const auto &dblCode : aztec_code_double_symbols) {
                 if (dblCode.c1 != c1 || dblCode.c2 != c2) {
                     continue;
                 }
@@ -578,9 +554,24 @@ BitVector AztecBarcode::bitStuffAndPad(const BitVector& input, int codeWordSize)
         res.appendBit(input.at(i++));
     }
 
-    // pad to nearest code word boundary
+    // check if we are code word aligned already
+    const auto trailingBits = res.size() % codeWordSize;
+    if (!trailingBits) { // nothing to pad
+        return res;
+    }
+
+    // pad with ones to nearest code word boundary
+    // last bit has to be zero if we'd otherwise would have all ones though
+    bool allOnes = true;
+    for (int i = res.size() - trailingBits; i < res.size(); ++i) {
+        allOnes &= res.at(i);
+    }
     while (res.size() % codeWordSize) {
-        res.appendBit(true);
+        if ((res.size() % codeWordSize) == (codeWordSize - 1)) {
+            res.appendBit(allOnes ? false : true);
+        } else {
+            res.appendBit(true);
+        }
     }
 
     return res;
